;*************************************************************
;* Blueprint LAN BIOS Header                  (BPBIOSHD.MOD) *
;* by Craig Chaiken                                          *
;* January 29, 1990                                          *
;*                                                           *
;* Defines all BP-LAN BIOS Constants.  Must be included in   *
;*   all BP-LAN system programs to maintain customizability. *
;*************************************************************
;
;*** Constants ***
;
soh     =       01h     ;start of header byte
ack     =       06h     ;acknowledge byte
nak     =       15h     ;negative acknowledge byte
can     =       18h     ;cancel byte
;
maxsockets      equ     20h     ;allocate space for 32 network sockets
;
ctrl_int        equ     80h     ;BIOS Control Interrupt
phys_int        equ     81h     ;Physical I/O Interrupt
link_int        equ     82h     ;Data Link Interrupt
netw_int        equ     83h     ;Network Interrupt
tran_int        equ     84h     ;Transport Data End to End Interrupt
sess_int        equ     85h     ;Session Control Interrupt
pres_int        equ     86h     ;Presentation Interrupt
appl_int        equ     87h     ;Application Interrupt

;*** BIOS Control Interrupt Subfunctions ***
get_nodes       equ     0
set_nodes       equ     1
attach_printer  equ     2
detach_printer  equ     3
install_port    equ     4
next_port       equ     5
install_appl    equ     6
get_info        equ     7

;*** Physical Interrupt Subfunctions ***
receive_status  equ     0
receive_byte    equ     1
transmit_status equ     2
transmit_byte   equ     3

;*** Data Link Layer Interrupt Subfunctions ***
clear_checksum  equ     0
calc_checksum   equ     1
get_checksum    equ     2
transmit_packet equ     3
receive_packet  equ     4
transmit_packet_status  equ     5
receive_packet_status   equ     6
get_retries     equ     7

;*** Network Layer Interrupt Subfunctions ***
;    Reserved for Customization

;*** Transport Layer Interrupt Subfunctions ***
;    Reserved for Customization

;*** Session Layer Interrupt Subfunctions ***
lock_printer    equ     0
unlock_printer  equ     1
set_semaphore   equ     2
get_semaphore   equ     3
clear_semaphore equ     4

;*** Presentation Layer Interrupt Subfunctions ***
;    Reserved for Customization

;*** Application Layer Interrupt Subfunctions ***
;    Reserved for Customization

;*** Macros for Accessing BPBIOS Functions ***
bpbios  macro   intnum,funcnum,socket_num,count,buffer_addr,overflow
        mov     ah,funcnum
        IFNB    <socket_num>
        mov     bl,socket_num
        ENDIF
        IFNB    <count>
        mov     cx,count
        ENDIF
        IFNB    <buffer_addr>
        mov     dx,buffer_addr overflow
        ENDIF
        int     intnum
        endm

get_packet      macro   socket_num,packet_length,packet_address,more
        mov     ah,receive_packet
        IFNB    <socket_num>
        mov     bl,socket_num
        ENDIF
        IFNB    <packet_address>
        mov     dx,packet_address more
        ENDIF
        int     link_int
        IFNB    <packet_length>
        mov     packet_length,cx
        ENDIF
        endm

put_packet      macro   socket_num,packet_length,packet_address,more
        mov     ah,transmit_packet
        IFNB    <socket_num>
        mov     bl,socket_num
        ENDIF
        IFNB    <packet_length>
        mov     cx,packet_length
        ENDIF
        IFNB    <packet_address>
        mov     dx,packet_address more
        ENDIF
        int     link_int
        endm

;*************************************************************
;* End of BPBIOSHD.MOD                                       *
;*************************************************************
