/************************************************************
* Blueprint LAN Turbo C Application Header          (BPC.H) *
* by Craig Chaiken                                          *
* October 25, 1989                                          *
*                                                           *
* Defines all BP-LAN BIOS Constants.  Must be included in   *
*   all BP-LAN system programs to maintain customizability. *
************************************************************/


/*** Architecture Layers ***/
#define	ctrl_int	0x80    /*BIOS Control Interrupt*/
#define	phys_int	0x81    /*Physical I/O Interrupt*/
#define	link_int	0x82    /*Data Link Interrupt*/
#define	netw_int	0x83    /*Network Interrupt*/
#define	tran_int	0x84    /*Transport Data End to End Interrupt*/
#define	sess_int	0x85    /*Session Control Interrupt*/
#define	pres_int	0x86    /*Presentation Interrupt*/
#define	appl_int	0x87    /*Application Interrupt*/

/**** BIOS Control Interrupt Subfunctions ****/
#define	get_nodes       0
#define	set_nodes       1
#define	attach_printer  2
#define	detach_printer  3
#define	install_port    4
#define	next_port       5
#define	install_appl    6
#define	get_info        7

/**** Physical Interrupt Subfunctions ****/
#define	receive_status  0
#define	receive_byte    1
#define	transmit_status 2
#define	transmit_byte   3

/**** Data Link Layer Interrupt Subfunctions ****/
#define	clear_checksum  0
#define	calc_checksum   1
#define	get_checksum    2
#define	transmit_packet 3
#define	receive_packet  4
#define	get_retries     5

char packet_buffer[517];
union REGS regs;
struct SREGS sregs;
char socket_number,option_number;
int packet_length;


void bpbios(char layer,char subfunction,char socket_number)
/***************************************************************/
/**** Perform BP-BIOS Call to Specified Architecture Layer, ****/
/****   Specified Subfunction, and Specified Socket Number  ****/
/***************************************************************/
{
	regs.h.ah=subfunction;
	regs.h.bl=socket_number;
	int86(layer,&regs,&regs);
}

int get_packet(char socket_number)
/*************************************************/
/**** Get Packet from Specified Socket Number ****/
/**** Store Packet in packet_buffer           ****/
/**** Return Packet Length                    ****/
/*************************************************/
{
	regs.h.ah=receive_packet;
	regs.h.bl=socket_number;
	regs.x.dx=FP_OFF(packet_buffer);
	sregs.ds=FP_SEG(packet_buffer);
	int86x(link_int,&regs,&regs,&sregs);
	return(regs.x.cx);
}

void put_packet(char socket_number)
/****************************************************/
/**** Transmit Packet to Specified Socket Number ****/
/**** Packet is found in Specified Buffer        ****/
/****************************************************/
{
	regs.h.ah=transmit_packet;
	regs.h.bl=socket_number;
	regs.x.dx=FP_OFF(packet_buffer);
	sregs.ds=FP_SEG(packet_buffer);
	regs.x.cx=packet_length;
	int86x(link_int,&regs,&regs,&sregs);
}


void exit_with_error(char errorlevel)
/******************************************************/
/**** Exit to DOS, and Return Specified Errorlevel ****/
/******************************************************/
{
	regs.h.ah=0x4c;	/**** DOS Exit Subfunction ****/
	regs.h.al=errorlevel;
	int86(0x21,&regs,&regs);
}

/************************************************************
* End of BPC.H                                              *
************************************************************/
