(************************************************************
* Blueprint LAN Pascal Application Header    (BPPASCAL.INC) *
* by Craig Chaiken                                          *
* October 24, 1989                                          *
*                                                           *
* Defines all BP-LAN BIOS Constants.  Must be included in   *
*   all BP-LAN system programs to maintain customizability. *
************************************************************)

Const

{*** Architecture Layers ***}
ctrl_int        =       $80;    {BIOS Control Interrupt}
phys_int        =       $81;    {Physical I/O Interrupt}
link_int        =       $82;    {Data Link Interrupt}
netw_int        =       $83;    {Network Interrupt}
tran_int        =       $84;    {Transport Data End to End Interrupt}
sess_int        =       $85;    {Session Control Interrupt}
pres_int        =       $86;    {Presentation Interrupt}
appl_int        =       $87;    {Application Interrupt}

{*** BIOS Control Interrupt Subfunctions ***}
get_nodes       =       0;
set_nodes       =       1;
attach_printer  =       2;
detach_printer  =       3;
install_port    =       4;
next_port       =       5;
install_appl    =       6;
get_info        =       7;

{*** Physical Interrupt Subfunctions ***}
receive_status  =       0;
receive_byte    =       1;
transmit_status =       2;
transmit_byte   =       3;

{*** Data Link Layer Interrupt Subfunctions ***}
clear_checksum  =       0;
calc_checksum   =       1;
get_checksum    =       2;
transmit_packet =       3;
receive_packet  =       4;
get_retries     =       5;

var
	packet_buffer:array[0..516] of byte;
	packet_length:integer;
     regs:registers;
	socket_number:byte;
	option_number:byte;

procedure bpbios(layer,subfunction,socket_number:byte);
{*************************************************************}
{*** Perform BP-BIOS Call to Specified Architecture Layer, ***}
{***   Specified Subfunction, and Specified Socket Number  ***}
{*************************************************************}
begin
	regs.ah:=subfunction;
	regs.bl:=socket_number;
	intr(layer,regs);
end;

function get_packet(socket_number:byte):integer;
{***********************************************}
{*** Get Packet from Specified Socket Number ***}
{*** Store Packet in packet_buffer           ***}
{*** Return Packet Length                    ***}
{***********************************************}
begin
     regs.ah:=receive_packet;
	regs.bl:=socket_number;
	regs.dx:=ofs(packet_buffer);
	regs.ds:=seg(packet_buffer);
	intr(link_int,regs);
	get_packet:=regs.cx
end;

procedure put_packet(socket_number:byte);
{**************************************************}
{*** Transmit Packet to Specified Socket Number ***}
{*** Packet is found in Specified Buffer        ***}
{**************************************************}
begin
     regs.ah:=transmit_packet;
	regs.bl:=socket_number;
	regs.cx:=packet_length;
	regs.dx:=ofs(packet_buffer);
	regs.ds:=seg(packet_buffer);
	intr(link_int,regs);
end;

function get_opt(option_number:integer):integer;
{*****************************************************}
{*** This function simulates the assembly language ***}
{*** procedure of the same name. It simply returns ***}
{*** the integer value of the specified option.    ***}
{*** This option is always prefixed with a '/'     ***}
{*****************************************************}
var
	s:string;
	v,code:integer;
begin
	s:=copy(paramstr(option_number),2,length(paramstr(option_number)));
	val(s,v,code);
	get_opt:=v
end;

procedure exit_with_error(errorlevel:byte);
{****************************************************}
{*** Exit to DOS, and Return Specified Errorlevel ***}
{****************************************************}
begin
	regs.ah:=$4c;	{*** DOS Exit Subfunction ***}
	regs.al:=errorlevel;
	msdos(regs);
end;

{************************************************************
* End of BPPASCAL.INC                                       *
************************************************************}
