;*************************************************************
;* Blueprint LAN Data Link Layer              (DATALINK.MOD) *
;* by Craig Chaiken                                          *
;* January 30, 1990                                          *
;*************************************************************

;*** Variables ***
;
temp    dw      ?
checksum        dw      0
packet_length   dw      0
retries db      0
socket_error    db      0
inta01  db      0
intb01  db      0

datalink_table  label   word
        org     datalink_table+2*clear_checksum
        dw      datalink1
        org     datalink_table+2*calc_checksum
        dw      datalink2
        org     datalink_table+2*get_checksum
        dw      datalink3
        org     datalink_table+2*transmit_packet
        dw      datalink4
        org     datalink_table+2*receive_packet
        dw      datalink5
        org     datalink_table+2*get_retries
        dw      datalink6
        org     datalink_table+16

datalink_interrupt      proc    far
        push    bx
        xor     bx,bx
        mov     bl,ah
        add     bx,bx
        jmp     cs:[datalink_table+bx]

datalink1:      pop     bx              ;clear checksum
        mov     cs:checksum,0
        iret

datalink2:      pop     bx              ;add AL to checksum
        xor     ah,ah
        add     cs:checksum,ax
        iret

datalink3:      pop     bx              ;get current checksum into AX
        mov     ax,cs:checksum
        iret

datalink4:      pop     bx              ;transmit packet at DS:SI
        call    out_packet
        iret

datalink5:      pop     bx              ;receive packet into DS:SI
        call    in_packet
        mov     cx,cs:packet_length
        iret

datalink6:      pop     bx              ;AL = packet retransmissions
        mov     al,cs:retries
        iret
datalink_interrupt      endp

;*** Block Input/Output Drivers ***
;
out_packet  proc    near    ;*** Transmit Packet via Current Socket ***
        push    ax
        in      al,21h
        mov     cs:inta01,al
        in      al,0a1h
        mov     cs:intb01,al
        mov     al,0ffh
        out     21h,al
        out     0a1h,al
        pop     ax
        mov     cs:retries,-1
trans1: inc     cs:retries
        call    put_header
        push    di
        push    cx
        bpbios  link_int,clear_checksum
        mov     al,cl
        bpbios  phys_int,transmit_byte
        mov     al,ch
        bpbios  phys_int,transmit_byte
        mov     di,dx
trans2: mov     al,[di]
        inc     di
        bpbios  phys_int,transmit_byte
        loop    trans2
        pop     cx
        pop     di
        call    send_checksum
        cmp     cs:socket_error,0
        jz      trans3
        jmp     trans4
trans3: cmp     al,ack
        jnz     trans1
trans4: push    ax
        mov     al,cs:inta01
        out     21h,al
        mov     al,cs:intb01
        out     0a1h,al
        pop     ax
        xor     ah,ah
        ret
out_packet      endp

in_packet       proc    near    ;*** Receive Packet ***
        in      al,21h
        mov     cs:inta01,al
        in      al,0a1h
        mov     cs:intb01,al
        mov     al,0ffh
        out     21h,al
        out     0a1h,al
        call    get_header
        push    di
        push    cx
        bpbios  link_int,clear_checksum
        bpbios  phys_int,receive_byte
        mov     cl,al
        bpbios  phys_int,receive_byte
        mov     ch,al
        mov     cs:packet_length,cx
        mov     di,dx
recei1: bpbios  phys_int,receive_byte
        mov     [di],al
        inc     di
        loop    recei1
        pop     cx
        pop     di
        call    verify_checksum
        cmp     cs:socket_error,0
        jz      recei2
        jmp     recei3
recei2: cmp     al,ack
        jnz     in_packet
recei3: push    ax
        mov     al,cs:inta01
        out     21h,al
        mov     al,cs:intb01
        out     0a1h,al
        pop     ax
        xor     ah,ah
        ret
in_packet       endp

put_header      proc    near    ;*** Send Packet Transmit Header ***
        push    ax
put_h1: mov     al,soh
        bpbios  phys_int,transmit_status
        jz      put_h2
        bpbios  phys_int,transmit_byte
put_h2: bpbios  phys_int,receive_status
        jz      put_h1
        bpbios  phys_int,receive_byte
        cmp     al,ack
        jnz     put_h1
        call    acknowledge
        pop     ax
        cli
        ret
put_header      endp

get_header      proc    near    ;*** Wait for Packet Transmit Header ***
        push    ax
get_h1: bpbios  phys_int,receive_byte
        cmp     al,soh                          ;check if character is SOH
        jnz     get_h1
get_h3: call    acknowledge                     ;if so, acknowledge SOH
get_h2: bpbios  phys_int,receive_byte           ;consume all duplicate SOHs
        cmp     al,soh
        jz      get_h2
        cmp     al,ack                          ;is SOH followed by ACK?
        jnz     get_h3                          ;no, then wait for retransmit
        pop     ax
        cli
        ret
get_header      endp

verify_checksum proc    near    ;*** Verify Packet Checksum ***
        push    cx
        bpbios  link_int,get_checksum
        mov     cs:temp,ax
        bpbios  phys_int,receive_byte
        mov     cl,al
        bpbios  phys_int,transmit_byte
        bpbios  phys_int,receive_byte
        mov     ch,al
        cmp     cx,cs:temp
        pop     cx
        sti
        jz      verif1
        jmp     neg_ack
verif1: jmp     acknowledge
verify_checksum endp

send_checksum   proc    near    ;*** Transmit Packet Checksum ***
        push    cx
        bpbios  link_int,get_checksum
        mov     cx,ax
        mov     al,cl
        bpbios  phys_int,transmit_byte
        bpbios  phys_int,receive_byte
        mov     al,ch
        bpbios  phys_int,transmit_byte
        bpbios  phys_int,receive_byte
        pop     cx
        sti
        ret
send_checksum   endp

acknowledge     proc    near    ;*** Acknowledge Reception of Packet ***
        mov     al,ack
        bpbios  phys_int,transmit_byte
        ret
acknowledge     endp

neg_ack proc    near            ;*** Negative Acknowledge, Bad Packet ***
        mov     al,nak
        bpbios  phys_int,transmit_byte
        ret
neg_ack endp

;*************************************************************
;* End of DATALINK.MOD                                       *
;*************************************************************
