;************************************************************
;* Semaphore Handling Module                 (SEMAPHOR.MOD) *
;* by Craig Chaiken                                         *
;* September 19, 1989                                       *
;*                                                          *
;* Function: Lock, Unlock or Clear Specified Semaphore      *
;************************************************************
;
semaphore_table dw      256 dup ('UU')  ;all semaphores unused

semaph  proc    near
        cmp     packet_buffer,'L'       ;lock semaphore
        jnz     semap1
        jmp     lock_sem
semap1: cmp     packet_buffer,'U'       ;unlock semaphore
        jnz     semap2
        jmp     unlock_sem
semap2: cmp     packet_buffer,'C'       ;clear semaphore
        jnz     semap3
        jmp     clear_sem
semap3: ret
semaph  endp

pre_sem proc    near    ;return semaphore status
        mov     packet_buffer,'D'       ;packet ID for Answered Request
        mov     packet_buffer+1,255     ;assume unsuccessful semaphore change
        lea     si,semaphore_table
        xor     bh,bh
        mov     bl,packet_buffer[1]     ;get semaphore number
        add     bx,bx                   ;build semaphore table offset
        ret
pre_sem endp

lock_sem        proc    near            ;lock semaphore if unused
        pushall
        call    pre_sem
        cmp     byte ptr [si+bx],'U'    ;Semaphore Unused?
        jnz     lock_1
        mov     packet_buffer[1],0      ;If so, indicate success
        mov     al,'L'                  ;  and lock semaphore
        mov     ah,packet_buffer[1]
        mov     [si+bx],ax
lock_1: put_packet      cs:socket_num,2,offset packet_buffer
        popall
        ret
lock_sem        endp

unlock_sem      proc    near    ;unlock semaphore if owned by this client
        pushall
        call    pre_sem
        mov     ax,[si+bx]
        mov     cl,'L'
        mov     ch,socket_num
        cmp     ax,cx                   ;Is semaphore locked by this client?
        jnz     unloc1
        mov     packet_buffer[1],0      ;If so, indicate success
        mov     al,'U'                  ;  and unlock semaphore
        mov     ah,packet_buffer[1]
        mov     [si+bx],ax
unloc1: put_packet      cs:socket_num,2,offset packet_buffer
        popall
        ret
unlock_sem      endp

clear_sem       proc    near    ;unlock semaphore regardless of current state
        pushall
        call    pre_sem
        mov     packet_buffer[1],0      ;Indicate success
        mov     al,'U'                  ;  and unlock semaphore
        mov     ah,packet_buffer[1]
        mov     [si+bx],ax
        put_packet      cs:socket_num,2,offset packet_buffer
        popall
        ret
clear_sem       endp

;************************************************************
;* End of Semaphore Handling Module  (SEMAPHOR.MOD)         *
;************************************************************
